@@ -51,10 +51,10 @@ add :: CliOpts -> Journal -> IO ()
 add opts j
     | f == "-" = return ()
     | otherwise = do
+  hPrintf stderr "Adding transactions to journal file \"%s\".\n" f
   hPutStrLn stderr $
-    "Enter one or more transactions, which will be added to your journal file.\n"
-    ++"To complete a transaction, enter . when prompted for an account.\n"
-    ++"To quit, press control-d or control-c."
+    "To complete a transaction, enter . (period) at an account prompt.\n"
+    ++"To stop adding transactions, enter . at a date prompt, or control-d/control-c."
   today <- getCurrentDay
   getAndAddTransactions j opts today
         `catch` (\e -> unless (isEOFError e) $ ioError e)
@@ -75,10 +75,12 @@ getTransaction :: Journal -> CliOpts -> Day
                     -> IO (Transaction,Day)
 getTransaction j opts defaultDate = do
   today <- getCurrentDay
-  datestr <- runInteractionDefault $ askFor "date" 
+  datestr <- runInteractionDefault $ askFor "date, or . to end"
             (Just $ showDate defaultDate)
-            (Just $ \s -> null s || 
-             isRight (parse (smartdate >> many spacenonewline >> eof) "" $ lowercase s))
+            (Just $ \s -> null s
+                         || s == "."
+                         || isRight (parse (smartdate >> many spacenonewline >> eof) "" $ lowercase s))
+  when (datestr == ".") $ ioError $ mkIOError eofErrorType "" Nothing Nothing
   description <- runInteractionDefault $ askFor "description" (Just "") Nothing
   let historymatches = transactionsSimilarTo j (patterns_ $ reportopts_ opts) description
       bestmatch | null historymatches = Nothing
@@ -117,9 +119,15 @@ getPostings st enteredps = do
                 | otherwise = Nothing
                 where Just ps = historicalps
       defaultaccount = maybe Nothing (Just . showacctname) bestmatch
-  account <- runInteraction j $ askFor (printf "account %d" n) defaultaccount (Just accept)
+      ordot | null enteredps || length enteredrealps == 1 = ""
+            | otherwise = ", or . to record"
+  account <- runInteraction j $ askFor (printf "account %d%s" n ordot) defaultaccount (Just accept)
   if account=="."
-    then return enteredps
+    then
+     if null enteredps
+      then do hPutStrLn stderr $ "\nPlease enter some postings first."
+              getPostings st enteredps
+      else return enteredps
     else do
       let defaultacctused = Just account == defaultaccount
           historicalps' = if defaultacctused then historicalps else Nothing
@@ -131,10 +139,16 @@ getPostings st enteredps = do
                            | n > 1             = Just balancingamountstr
                            | otherwise         = Nothing
               where
-                -- force a decimal point in the output in case there's a
-                -- digit group separator that would be mistaken for one
-                historicalamountstr = showMixedAmountWithPrecision maxprecisionwithpoint $ pamount $ fromJust bestmatch'
-                balancingamountstr  = showMixedAmountWithPrecision maxprecisionwithpoint $ negate $ sum $ map pamount enteredrealps
+                historicalamountstr = showMixedAmountWithPrecision p $ pamount $ fromJust bestmatch'
+                balancingamountstr  = showMixedAmountWithPrecision p $ negate $ sum $ map pamount enteredrealps
+                -- what should this be ?
+                -- 1 maxprecision (show all decimal places or none) ?
+                -- 2 maxprecisionwithpoint (show all decimal places or .0 - avoids some but not all confusion with thousands separators) ?
+                -- 3 canonical precision for this commodity in the journal ?
+                -- 4 maximum precision entered so far in this transaction ?
+                -- 5 3 or 4, whichever would show the most decimal places ?
+                -- I think 1 or 4, whichever would show the most decimal places
+                p = maxprecisionwithpoint
       amountstr <- runInteractionDefault $ askFor (printf "amount  %d" n) defaultamountstr validateamount
       let amount  = fromparse $ runParser (someamount <|> return missingamt) ctx     "" amountstr
           amount' = fromparse $ runParser (someamount <|> return missingamt) nullctx "" amountstr
@@ -191,26 +205,24 @@ askFor prompt def validator = do
 journalAddTransaction :: Journal -> CliOpts -> Transaction -> IO Journal
 journalAddTransaction j@Journal{jtxns=ts} opts t = do
   let f = journalFilePath j
-  appendToJournalFile f $ showTransaction t
+  appendToJournalFileOrStdout f $ showTransaction t
   when (debug_ opts) $ do
     putStrLn $ printf "\nAdded transaction to %s:" f
     putStrLn =<< registerFromString (show t)
   return j{jtxns=ts++[t]}
 
--- | Append data to a journal file; or if the file is "-", dump it to stdout.
-appendToJournalFile :: FilePath -> String -> IO ()
-appendToJournalFile f s =
-    if f == "-"
-    then putStr $ sep ++ s
-    else appendFile f $ sep++s
-    where 
-      -- appendFile means we don't need file locking to be
-      -- multi-user-safe, but also that we can't figure out the minimal
-      -- number of newlines needed as separator
-      sep = "\n\n"
-      -- sep | null $ strip t = ""
-      --     | otherwise = replicate (2 - min 2 (length lastnls)) '\n'
-      --     where lastnls = takeWhile (=='\n') $ reverse t
+-- | Append a string, typically one or more transactions, to a journal
+-- file, or if the file is "-", dump it to stdout.  Tries to avoid
+-- excess whitespace.
+appendToJournalFileOrStdout :: FilePath -> String -> IO ()
+appendToJournalFileOrStdout f s
+  | f == "-"  = putStr s'
+  | otherwise = appendFile f s'
+  where s' = "\n" ++ ensureOneNewlineTerminated s
+
+-- | Replace a string's 0 or more terminating newlines with exactly one.
+ensureOneNewlineTerminated :: String -> String
+ensureOneNewlineTerminated = (++"\n") . reverse . dropWhile (=='\n') . reverse
 
 -- | Convert a string of journal data into a register report.
 registerFromString :: String -> IO String
