@@ -147,7 +147,11 @@ This implementation turned out to be a bit convoluted but implements the followi
 -- | Render one balance report line item as plain text.
 accountsReportItemAsText :: ReportOpts -> [FormatString] -> AccountsReportItem -> [String]
 accountsReportItemAsText opts format (_, accountName, depth, Mixed amounts) =
-    case amounts of
+    -- 'amounts' could contain several quantities of the same commodity with different price.
+    -- In order to combine them into single value (which is expected) we take the first price and
+    -- use it for the whole mixed amount. This could be suboptimal. XXX
+    let Mixed normAmounts = normaliseMixedAmountPreservingFirstPrice (Mixed amounts) in
+    case normAmounts of
       [] -> []
       [a] -> [formatAccountsReportItem opts (Just accountName) depth a format]
       (as) -> multiline as
