@@ -1,3 +1,4 @@
+{-# LANGUAGE TemplateHaskell #-}
 {-|
 
 Command-line options for the hledger program, and option-parsing utilities.
@@ -25,9 +26,6 @@ import Hledger.Cli.Format as Format
 import Hledger.Cli.Version
 
 
-progname = "hledger"
-progversion = progversionstr progname
-
 -- 1. cmdargs mode and flag definitions, for the main and subcommand modes.
 -- Flag values are parsed initially to simple RawOpts to permit reuse.
 
@@ -428,10 +426,10 @@ defaultBalanceFormatString = [
 journalFilePathFromOpts :: CliOpts -> IO String
 journalFilePathFromOpts opts = do
   f <- myJournalPath
-  return $ errorIfContainsTilde $ fromMaybe f $ file_ opts
-
-errorIfContainsTilde s |'~' `elem` s = error' "unsupported literal ~ found in environment variable, please adjust"
-                       | otherwise   = s
+  let f' = fromMaybe f $ file_ opts
+  if '~' `elem` f'
+   then error' $ printf "~ in the journal file path is not supported, please adjust (%s)" f'
+   else return f'
 
 aliasesFromOpts :: CliOpts -> [(AccountName,AccountName)]
 aliasesFromOpts = map parseAlias . alias_
@@ -445,7 +443,7 @@ aliasesFromOpts = map parseAlias . alias_
             alias' = case alias of ('=':rest) -> rest
                                    _ -> orig
 
-showModeHelp = showText defaultWrap . helpText HelpFormatDefault
+showModeHelp = showText defaultWrap . helpText [] HelpFormatDefault
 
 tests_Hledger_Cli_Options = TestList
  [
